@@ -22,6 +22,7 @@ package Lingua::NATools::PTD;
 
 use base 'Exporter';
 our @EXPORT = 'toentry';
+our @EXPORT_OK = qw/bws ucts/;
 use warnings;
 use strict;
 
@@ -37,6 +38,8 @@ use Lingua::NATools::PTD::SQLite;
 
 our $VERSION = '1.1';
 
+=encoding UTF-8
+
 =head1 NAME
 
 Lingua::NATools::PTD - Module to handle PTD files in Dumper Format
@@ -58,7 +61,7 @@ manipulate it.
 The C<new> constructor returns a new Lingua::NATools::PTD object. This constructor
 receives a PTD file in dumper format.
 
-  $ptd = Lingua::NATools::PTD->new( $ptd_file );
+  my $ptd = Lingua::NATools::PTD->new( $ptd_file );
 
 If the filename matches with C<< /dmp.bz2$/ >> (that is, ends in
 dmp.bz2) it is considered to be a bzip2 file and will be decompressed
@@ -76,7 +79,7 @@ variables. Following options are recognized:
 
 Sets verbosity.
 
-  $ptd = Lingua::NATools::PTD->new( $ptd_file, verbose => 1 );
+  my $ptd = Lingua::NATools::PTD->new( $ptd_file, verbose => 1 );
 
 =back
 
@@ -131,7 +134,7 @@ their probability (starting with higher probabilities).
 
 The format is Perl code, and thus, can be used independetly of this module.
 
-   $ptd -> dump;
+   $ptd->dump;
 
 Note that the C<dump> method writes to the Standard Output stream.
 
@@ -166,7 +169,7 @@ The C<words> method returns an array (not a reference) to the list of
 words of the dictionary: its domain. Pass a true value as argument and
 the list is returned sorted.
 
-   @words = $ptd -> words;
+   my @words = $ptd->words;
 
 =cut
 
@@ -185,7 +188,7 @@ sub words {
 The C<trans> method receives a word, and returns the list of its possible
 translations.
 
-   @translations = $ptd -> trans( "dog" );
+   my @translations = $ptd->trans( "dog" );
 
 =cut
 
@@ -200,13 +203,24 @@ sub trans {
 }
 
 
+=head2 C<exists>
+
+Checks if a word is in a dictionary
+
+=cut
+
+sub exists {
+    my ($self, $word ) = @_;
+    return exists $self->{$word};
+}
+
 =head2 C<transHash>
 
 The C<transHash> method receives a word, and returns an hash where
 keys are the its possible translations, and values the corresponding
 translation probabilities.
 
-   %trans = $ptd -> transHash( "dog" );
+   my %trans = $ptd->transHash( "dog" );
 
 Returns the empty hash if the word does not exist.
 
@@ -226,7 +240,7 @@ sub transHash {
 The C<prob> method receives a word and a translation, and returns the
 probability of that word being translated that way.
 
-   $probability = $ptd -> prob ( "cat", "gato");
+   my $probability = $ptd->prob("cat", "gato");
 
 =cut
 
@@ -241,7 +255,7 @@ sub prob {
 Returns the total number of words from the source-corpus that originated
 the PTD. Basically, the sum of the C<count> attribute for all words.
 
-   $size = ptd -> size;
+   my $size = $ptd->size;
 
 =cut
 
@@ -254,7 +268,7 @@ sub size {
 The C<count> method receives a word and returns the occurrence count for
 that word.
 
-   $count = $ptd -> count ("cat");
+   my $count = $ptd->count("cat");
 
 If no argument is supplied, returns the total dictionary count (sum of
 all words).
@@ -369,6 +383,58 @@ sub subtractDomain {
 }
 
 
+
+=head2 C<restrictDomain>
+
+Domain restrict function: interface is similar to subtractDomain function
+
+This method restricts the domain of a PTD to a set of elements. 
+This set can be defines as another PTD (domain is
+used), as a Perl array reference, as a Perl hash reference (domain is
+used) or as a Perl array (not reference). Returns the dictionary after
+domain restriction takes place.
+
+  # removes portuguese articles from the dictionary
+  $ptd->subtractDomain( qw.o a os as. );
+
+  # removes a set of stop words from the dictionary
+  $ptd->subtractDomain( \@stopWords );
+
+  # removes the words present on other_ptd from ptd
+  $ptd->subtractDomain( $other_ptd );
+
+=cut
+
+sub restrictDomain {
+    my ($self, $other, @more) = @_;
+	
+    my @domain;
+    if (ref($other) =~ /Lingua::NATools::PTD/ and $other->isa("Lingua::NATools::PTD")) {
+        @domain = $other->words;
+    }
+    elsif (ref($other) eq "ARRAY") {
+        @domain = @$other
+    }
+    elsif (ref($other) eq "HASH") {
+        @domain = keys %$other
+    }
+    else {
+        @domain = ($other, @more);
+    }
+    my %domain;
+    @domain{@domain} = @domain;
+	
+    $self -> downtr (
+                     sub {
+                         my ($w,$c,%t) = @_;
+                         return exists($domain{$w}) ? toentry($w,$c,%t):undef
+                     },
+                     filter => 1,
+                    );
+    $self->_calculate_sizes();
+    return $self;
+}
+
 =head2 C<reprob>
 
 This method recalculates all probabilities accordingly with the number
@@ -531,14 +597,14 @@ as argument. This function will receive, in each call, the word in the
 source language, the number of occurrences, and the hash of
 translations.
 
-  $ptd -> downtr( sub { my ($w,$c,%t) = @_;
-                        if ($w =~ /[^A-Za-z0-9]/) {
-                            return undef;
-                        } else {
-                            return toentry($w,$c,%t);
-                        }
-                },
-               filter => 1);
+  $ptd->downtr( sub { my ($w,$c,%t) = @_;
+                      if ($w =~ /[^A-Za-z0-9]/) {
+                          return undef;
+                      } else {
+                          return toentry($w,$c,%t);
+                      }
+              },
+             filter => 1);
 
 Set the filter flag if your downtr function is replacing the original
 dictionary.
@@ -704,6 +770,315 @@ sub lowercase {
                  );
 }
 
+=head2 C<ucts>
+
+Create unambiguous-concept traslation sets.
+
+  my $result =  ucts($ptd1, $ptd2, m=>0.1, M=>0.8);
+
+Available options are:
+
+=over 4
+
+=item C<m>
+
+Mininum number of occurences of each token. Must be an
+integer (default: 10).
+
+=item C<M>
+
+Manixum number of occurences of each token. Must be an
+integer (default: 100).
+
+=item C<p>
+
+Minimum probabilty for translation. Must be a probability
+in the interval [0,1] (default: 0.2).
+
+=item C<P>
+
+Minimum probabilty for the inverse translations. Must be a
+probability in the interval [0,1] (default: 0.8).
+
+=item C<r=0|1>
+
+Print rank (default: 0).
+
+=item C<pp=0|1>
+
+Pretty print output (default: 0).
+
+=item C<output=filename>
+
+Pretty print output to file C<filename>.
+
+=back
+
+=cut
+
+sub ucts {
+    my ($fileA, $fileB, %my_opts) = @_;
+
+    my $min_occur = $my_opts{m} || 10;
+    my $max_occur = $my_opts{M} || 100;
+    my $prob = $my_opts{p} || 0.2;
+    my $probi = $my_opts{P} || 0.8;
+    my $rank = $my_opts{r} || 0;
+    my $pp = $my_opts{pp} || 0;
+    my $output = $my_opts{output} || '';
+
+    # check files exist
+    unless ($fileA and $fileB) {
+       die "Error: need at least two PTDs given as argument.";
+    }
+
+    # handle output handles
+    $pp = 1 if $output;
+    open STDOUT, '>', $output if $output;
+    binmode(STDOUT, ':utf8'); # XXX
+
+    # load PTDs
+    my $ptd;
+    if (ref($fileA) =~ m/^Lingua::NATools::PTD/) {
+        $ptd = $fileA;
+    }
+    else {
+       if (-e $fileA) {
+           $ptd = Lingua::NATools::PTD->new($fileA);
+       }
+       else {
+           die "Error: file not found: $_";
+       }
+    }
+    my $ptd_inv;
+    if (ref($fileB) =~ m/^Lingua::NATools::PTD/) {
+        $ptd_inv = $fileB;
+    }
+    else {
+       if (-e $fileB) {
+           $ptd_inv = Lingua::NATools::PTD->new($fileB);
+       }
+       else {
+           die "Error: file not found: $_";
+       }
+    }
+
+    if ($pp and $fileA =~ m/.*?(\w\w)\-(\w\w)/) { # XXX
+        print "Langs: $1, $2\n" if $pp;
+    }
+
+    my (%left, %right);
+
+    # process each word in the PTD
+    my @words = $ptd->words;
+    foreach (@words) {
+        my $r = __build_ucts($ptd, $ptd_inv, $min_occur, $max_occur, $prob, $probi, $_);
+        $left{$_} = $r if $r;
+    }
+    # process each word in the inverse PTD
+    @words = $ptd_inv->words;
+    foreach (@words) {
+        my $r = __build_ucts($ptd_inv, $ptd, $min_occur, $max_occur, $prob, $probi, $_);
+        $right{$_} = $r if $r;
+    }
+
+    my @final = ();
+    foreach my $l (keys %left) {
+        my %ll = ($l=>1);
+        my %rr;
+        $rr{$_}++ for @{$left{$l}->{trans}};
+        my $rank = $left{$l}->{rank};
+
+        foreach (@{$left{$l}->{trans}}) {
+            $rr{$_}++;
+            if (exists($right{$_})) {
+                $ll{$_}++ for @{$right{$_}->{trans}};
+                delete $right{$_};
+            }
+        }
+        push @final, {l=>[keys %ll], r=>[keys %rr], rank=>$rank};
+    }
+    foreach my $r (keys %right) {
+        my %ll;
+        my %rr = ($r=>1);;
+        $ll{$_}++ for @{$right{$r}->{trans}};
+        my $rank = $right{$r}->{rank};
+
+        push @final, {l=>[keys %ll], r=>[keys %rr], rank=>$rank};
+    }
+
+    if ($pp) {
+        __pp_ucts($_,$rank) foreach (@final);
+    }
+    else {
+        return [@final];
+    }
+}
+
+sub __build_ucts {
+   my ($ptd, $ptd_inv, $min_occur, $max_occur, $prob, $probi, $word) = @_;
+
+   my $count = $ptd->count($word); ##  or print STDERR "### $word\n";
+   $count //= 0;
+   return undef unless ($min_occur <= $count and $count <= $max_occur);
+
+   my $total = 0;
+   my %trans = ();
+   my %transHash = $ptd->transHash($word);
+
+   foreach (keys %transHash) {
+      my $p = $transHash{$_};
+      next unless ($p >= $prob);
+      my $p_inv = $ptd_inv->prob($_, $word);
+      next unless ($p_inv >= $probi);
+
+      my $counti = $ptd_inv->count($_);
+      if ( ($min_occur <= $counti) and ($counti <= $max_occur) ) {
+         if ($total) { $total = ($total+$p+$p_inv)/2; }
+         else { $total = $p+$p_inv; }
+
+         $trans{$_}++;
+      }
+   }
+
+   return undef unless %trans;
+   return {trans=>[keys %trans], rank=>$total};
+}
+
+=head2 C<bws>
+
+Create bi-words sets given a PTD pair.
+
+  my $result = bws($ptd1, $ptd2, m=>0.1, p=>0.4);
+
+C<$ptd1> and C<$ptd2> can be filenames for the PTDs or already create
+PTD objects.
+
+The following options are available:
+
+=over 4
+
+=item C<m>
+
+Mininum number of occurences of each token. Must be an integer
+(default: 10).
+
+=item C<p>
+
+Minimum probabilty for translation. Must be a probability
+in the interval [0,1] (default: 0.4).
+
+=item C<r=0|1>
+
+Print rank (default: 0).
+
+=item C<pp=0|1>
+
+Pretty print output (default: 0).
+
+=item C<output=filename>
+
+Pretty print output to file C<filename>.
+
+=back
+
+=cut
+
+sub bws {
+    my ($fileA, $fileB, %my_opts) = @_;
+
+    my $min_occur = $my_opts{m} || 10;
+    my $prob = $my_opts{p} || 0.4;
+    my $rank = $my_opts{r} || 0;
+    my $pp = $my_opts{pp} || 0;
+    my $output = $my_opts{output} || '';
+
+    # check files exist
+    unless ($fileA and $fileB) {
+       die "Error: need at least two PTDs given as argument.";
+    }
+
+    # handle output handles
+    $pp = 1 if $output;
+    open STDOUT, '>', $output if $output;
+    binmode(STDOUT, ':utf8'); # XXX
+
+    # load PTDs
+    my $ptd;
+    if (ref($fileA) =~ m/^Lingua::NATools::PTD/) {
+        $ptd = $fileA;
+    }
+    else {
+       if (-e $fileA) {
+           $ptd = Lingua::NATools::PTD->new($fileA);
+       }
+       else {
+           die "Error: file not found: $_";
+       }
+    }
+    my $ptd_inv;
+    if (ref($fileB) =~ m/^Lingua::NATools::PTD/) { 
+        $ptd_inv = $fileB; 
+    } 
+    else { 
+       if (-e $fileB) {
+           $ptd_inv = Lingua::NATools::PTD->new($fileB);
+       }
+       else {
+           die "Error: file not found: $_";
+       }
+    }
+
+    if ($pp and $fileA =~ m/.*?(\w\w)\-(\w\w)/) { # XXX
+        print "Langs: $1, $2\n" if $pp;
+    }
+
+    my @final;
+
+    my @words = $ptd->words;
+    foreach my $word (@words) {
+        my $count = $ptd->count($_);
+        next unless ($count >= $min_occur);
+
+        my %transHash = $ptd->transHash($word);
+        foreach (keys %transHash) {
+            my $p = $transHash{$_};
+            next unless ($p >= $prob);
+
+            __pp_ucts({l=>[$word],r=>[$_],rank=>$p}, $rank) if $pp;
+            push @final, {l=>$word, r=>$_, rank=>$p} unless $pp;
+        }
+    }
+    @words = $ptd_inv->words;
+    foreach my $word (@words) {
+        my $count = $ptd->count($_);
+        next unless ($count >= $min_occur);
+
+        my %transHash = $ptd->transHash($word);
+        foreach (keys %transHash) {
+            my $p = $transHash{$_};
+            next unless ($p >= $prob);
+
+            __pp_ucts({l=>[$_],r=>[$word],rank=>$p}, $rank) if $pp;
+            push @final, {l=>$_, r=>$word, rank=>$p} unless $pp;
+        }
+    }
+
+    close STDOUT;
+    return [@final] unless $pp;
+}
+
+sub __pp_ucts {
+    my ($r, $rank) = @_;
+
+    if ($rank) {
+        printf "[%f]%s=%s\n", $r->{rank}, (join ',', @{$r->{l}}), join ',', @{$r->{r}};
+    }
+    else {
+        printf "%s=%s\n", (join ',', @{$r->{l}}), join ',', @{$r->{r}};
+    }
+}
+
 =head1 SEE ALSO
 
 NATools, perl(1)
